@@ -16,39 +16,60 @@
 
 """Recursive diff and patch for nested structures."""
 
-from difflib import SequenceMatcher
 from pickle import dumps
 from warnings import warn
 
+import nested_diff.handlers
+
 __all__ = ['Differ', 'Iterator', 'Patcher', 'diff', 'patch']
 
-__version__ = '0.9'
+__version__ = '0.10'
 __author__ = 'Michael Samoglyadov'
 __license__ = 'Apache License, Version 2.0'
 __website__ = 'https://github.com/mr-mixas/Nested-Diff.py'
 
+_DEFAULT_HANDLER = nested_diff.handlers.TypeHandler()
+
+# Temporary handlers index
+# TODO: turn it to a tuple and use as default handlers after deprecation cycle
+_TYPE_HANDLERS = {
+    dict: nested_diff.handlers.DictHandler(),
+    list: nested_diff.handlers.ListHandler(),
+    tuple: nested_diff.handlers.TupleHandler(),
+    set: nested_diff.handlers.SetHandler(),
+    frozenset: nested_diff.handlers.FrozenSetHandler(),
+}
+
 
 class Differ(object):
     """
     Compute recursive diff for two passed objects.
 
-    Resulting diff is a dict and may contain following keys:
+    Dicts, lists, tuples, sets and frozensets traversed recursively, other
+    types compared by values. Any type diff may be customized by user-defined
+    type handlers.
+
+    Diff is a dict and may contain status keys:
+
     `A` stands for 'added', it's value - added item.
     `D` means 'different' and contains subdiff.
-    `E` diffed entity (optional), value - empty instance of entity's class.
-    `I` index for sequence item, used only when prior item was omitted.
     `N` is a new value for changed item.
     `O` is a changed item's old value.
     `R` key used for removed item.
     `U` represent unchanged item.
 
+    and auxiliary keys:
+
+    `C` comment; optional, value - arbitrary string.
+    `E` diffed entity (optional), value - empty instance of entity's class.
+    `I` index for sequence item, used only when prior item was omitted.
+
     Diff metadata alternates with actual data; simple types specified as is,
     dicts, lists and tuples contain subdiffs for their items with native for
-    such types addressing: indexes for lists and tuples and keys for
-    dictionaries. Any status type, except `D`, `E` and `I` may be omitted
-    during diff computation. `E` tag is used with `D` when entity unable to
-    contain diff by itself (set, frozenset); `D` contain a list of subdiffs
-    in this case.
+    such types addressing: indexes for lists and tuples, keys for dictionaries.
+    Any status key, except `D` may be omitted during diff computation. `E` key
+    is used with `D` when entity unable to contain diff by itself (set,
+    frozenset for example); `D` contain a list of subdiffs in this case.
 
     Example:
     a:  {"one": [5,7]}
@@ -75,16 +96,14 @@ class Differ(object):
     | +- changes somewhere deeply inside
     +- diff is always a dict
 
-    Dicts, lists, sets and tuples traversed recursively, all other types
-    compared by values.
-
     """
 
+    default_handler = _DEFAULT_HANDLER
+
     def __init__(self, A=True, N=True, O=True, R=True, U=True,  # noqa: E501 E741 N803
-                 trimR=False, diff_method=None, text_diff_ctx=-1,
-                 multiline_diff_context=-1):
+                 trimR=False, text_diff_ctx=-1, handlers=()):
         """
-        Construct Differ.
+        Initialize Differ.
 
         Optional arguments:
         `A`, `N`, `O`, `R`, `U` are toggles for according diff ops and all
@@ -93,19 +112,13 @@ class Differ(object):
         `trimR` when True will drop (replace by `None`) removed data from diff;
         default is `False`.
 
-        `diff_method` method with such name (if object have one) from first
-        diffed object will be called for diff. Second diffed object and current
-        differ will be passed as arguments, diff expected for output.
-        Disabled (`None`) by default. This option is deprecated and will be
-        removed in the next release.
-
         `text_diff_ctx` defines amount of context lines for text (multiline
-        strings) diffs, disabled entirely when value is negative.
+        strings) diffs, disabled entirely when value is negative. This opt is
+        deprecated and will be removed in the next release.
+
+        `handlers` is a list of type handlers.
 
         """
-        self.__diff_method = diff_method
-        self.lcs = SequenceMatcher(isjunk=None, autojunk=False)
-
         self.op_a = A
         self.op_n = N
         self.op_o = O
@@ -121,19 +134,18 @@ class Differ(object):
             tuple: self.diff_tuple,
         }
 
-        if multiline_diff_context >= 0:
-            warn('`multiline_diff_context` is deprecated and will be removed'
-                 ' in the next release, use `text_diff_ctx` instead',
-                 DeprecationWarning, stacklevel=2)
-            if text_diff_ctx < 0:
-                text_diff_ctx = multiline_diff_context
-            else:
-                raise TypeError('`multiline_diff_context` and `text_diff_ctx`'
-                                ' can not be used together')
+        self.__handlers = {}
+
+        for handler in handlers:
+            self.set_handler(handler)
 
         if text_diff_ctx >= 0 and self.op_n and self.op_o:
+            warn('`text_diff_ctx` opt is deprecated and will be removed in'
+                 ' the next release', DeprecationWarning, stacklevel=2)
+
+            self.__handlers[str] = nested_diff.handlers.TextHandler(
+                context=text_diff_ctx)
             self.__differs[str] = self.diff_text
-            self.text_diff_ctx = text_diff_ctx
 
     def diff(self, a, b):
         """
@@ -147,34 +159,23 @@ class Differ(object):
         :param b: Second object to diff.
 
         """
-        if self.__diff_method is not None:
-            warn('diff internal methods are deprecated and will be removed in'
-                 ' the next release', DeprecationWarning, stacklevel=2)
-            try:
-                return a.__getattribute__(self.__diff_method)(b, self)
-            except AttributeError:
-                pass
-
         if a.__class__ is b.__class__:
             # it's faster to compare pickled dumps and dig differences
             # afterwards than recursively diff each pair of objects
             if a is b or dumps(a, -1) == dumps(b, -1):
                 return {'U': a} if self.op_u else {}
 
+            # TODO: get rid of `get_differ` and get funcs directly
             return self.get_differ(a.__class__)(a, b)
 
         return self.diff__default(a, b)
 
     def diff__default(self, a, b):
         """Return default diff."""
-        dif = {}
+        warn('`diff__default` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        if self.op_n:
-            dif['N'] = b
-        if self.op_o:
-            dif['O'] = a
-
-        return dif
+        return self.default_handler.diff(self, a, b)
 
     def diff_dict(self, a, b):
         """
@@ -183,38 +184,11 @@ class Differ(object):
         :param a: First dict to diff.
         :param b: Second dict to diff.
 
-        >>> a = {'one': 1, 'two': 2, 'three': 3}
-        >>> b = {'one': 1, 'two': 42}
-        >>>
-        >>> Differ(O=False, U=False).diff_dict(a, b)
-        {'D': {'three': {'R': 3}, 'two': {'N': 42}}}
-        >>>
-
         """
-        dif = {}
+        warn('`diff_dict` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for key in set(a).union(b):
-            try:
-                old = a[key]
-                try:
-                    new = b[key]
-                except KeyError:  # removed
-                    if self.op_r:
-                        dif[key] = {'R': None if self.op_trim_r else old}
-                    continue
-            except KeyError:  # added
-                if self.op_a:
-                    dif[key] = {'A': b[key]}
-                continue
-
-            subdiff = self.diff(old, new)
-            if subdiff:
-                dif[key] = subdiff
-
-        if dif:
-            return {'D': dif}
-
-        return dif
+        return _TYPE_HANDLERS[dict].diff(self, a, b)
 
     def diff_list(self, a, b):
         """
@@ -223,66 +197,11 @@ class Differ(object):
         :param a: First list to diff.
         :param b: Second list to diff.
 
-        >>> a = [0,1,2,3]
-        >>> b = [  1,2,4,5]
-        >>>
-        >>> Differ(O=False, U=False).diff_list(a, b)
-        {'D': [{'R': 0}, {'N': 4, 'I': 3}, {'A': 5}]}
-        >>>
-
         """
-        self.lcs.set_seq1(tuple(dumps(i, -1) for i in a))
-        self.lcs.set_seq2(tuple(dumps(i, -1) for i in b))
+        warn('`diff_list` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        dif = []
-        i = j = 0
-        force_index = False
-
-        for ai, bj, _ in self.lcs.get_matching_blocks():
-            while i < ai and j < bj:
-                subdiff = self.diff(a[i], b[j])
-                if subdiff:
-                    dif.append(subdiff)
-                    if force_index:
-                        dif[-1]['I'] = i
-                        force_index = False
-                else:
-                    force_index = True
-
-                i += 1
-                j += 1
-
-            while i < ai:  # removed
-                if self.op_r:
-                    dif.append({'R': None if self.op_trim_r else a[i]})
-                    if force_index:
-                        dif[-1]['I'] = i
-                        force_index = False
-                else:
-                    force_index = True
-
-                i += 1
-
-            while j < bj:  # added
-                if self.op_a:
-                    dif.append({'A': b[j]})
-                    if force_index:
-                        dif[-1]['I'] = i
-                        force_index = False
-                else:
-                    force_index = True
-
-                j += 1
-
-        if dif:
-            return {'D': dif}
-
-        return {}
-
-    def diff_multiline(self, a, b):
-        warn('Differ.diff_multiline method is deprecated and will be removed'
-             'in the next release', DeprecationWarning, stacklevel=2)
-        return self.diff_text(self, a, b)
+        return _TYPE_HANDLERS[list].diff(self, a, b)
 
     def diff_text(self, a, b):
         r"""
@@ -294,46 +213,11 @@ class Differ(object):
         :param a: First string to diff.
         :param b: Second string to diff.
 
-        >>> a = 'one'
-        >>> b = 'one\ntwo'
-        >>>
-        >>> Differ(text_diff_ctx=3).diff_text(a, b)
-        {'D': [{'I': [0, 1, 0, 2]}, {'U': 'one'}, {'A': 'two'}], 'E': ''}
-
         """
-        lines_a = a.split('\n', -1)
-        lines_b = b.split('\n', -1)
+        warn('`diff_text` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        if len(lines_a) == len(lines_b) == 1:
-            return self.diff__default(a, b)
-
-        dif = []
-        self.lcs.set_seq1(lines_a)
-        self.lcs.set_seq2(lines_b)
-
-        for group in self.lcs.get_grouped_opcodes(self.text_diff_ctx):
-            dif.append({
-                'I': [
-                    group[0][1], group[-1][2],
-                    group[0][3], group[-1][4],
-                ],
-            })
-
-            for op, i1, i2, j1, j2 in group:
-                if op == 'equal':
-                    dif.extend({'U': line} for line in lines_a[i1:i2])
-                    continue
-
-                if op != 'insert':
-                    dif.extend({'R': line} for line in lines_a[i1:i2])
-
-                if op != 'delete':
-                    dif.extend({'A': line} for line in lines_b[j1:j2])
-
-        if dif:
-            return {'D': dif, 'E': a.__class__()}
-
-        return {}
+        return self.__handlers[str].diff(self, a, b)
 
     def diff_set(self, a, b):
         """
@@ -342,32 +226,11 @@ class Differ(object):
         :param a: First set to diff.
         :param b: Second set to diff.
 
-        >>> a = {1, 2}
-        >>> b = {2, 3}
-        >>>
-        >>> Differ(U=False).diff_set(a, b)
-        {'D': [{'R': 1}, {'A': 3}], 'E': set()}
-        >>>
-
         """
-        dif = []
+        warn('`diff_set` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for i in a.union(b):
-            if i in a:
-                if i in b:
-                    if self.op_u:
-                        dif.append({'U': i})
-                elif self.op_r:
-                    # ignore trimR opt here: value required for removal
-                    dif.append({'R': i})
-            else:  # added
-                if self.op_a:
-                    dif.append({'A': i})
-
-        if dif:
-            return {'D': dif, 'E': a.__class__()}
-
-        return {}
+        return _TYPE_HANDLERS[set].diff(self, a, b)
 
     def diff_tuple(self, a, b):
         """
@@ -376,22 +239,11 @@ class Differ(object):
         :param a: First tuple to diff.
         :param b: Second tuple to diff.
 
-        >>> a = (  1,2,4,5)
-        >>> b = (0,1,2,3)
-        >>>
-        >>> Differ(O=False, U=False).diff_tuple(a, b)
-        {'D': ({'A': 0}, {'N': 3, 'I': 2}, {'R': 5})}
-        >>>
-
         """
-        dif = self.diff_list(a, b)
+        warn('`diff_tuple` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        try:
-            dif['D'] = tuple(dif['D'])
-        except KeyError:
-            pass
-
-        return dif
+        return _TYPE_HANDLERS[tuple].diff(self, a, b)
 
     def get_differ(self, type_):
         """
@@ -400,6 +252,9 @@ class Differ(object):
         :param type_: diffed object type.
 
         """
+        warn('`get_differ` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         try:
             return self.__differs[type_]
         except KeyError:
@@ -413,35 +268,50 @@ class Differ(object):
         :param method: diff method.
 
         """
+        warn('`set_differ` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         self.__differs[type_] = method
 
+    def set_handler(self, handler):
+        """
+        Set handler.
+
+        :param handler: handlers.TypeHandler.
+
+        """
+        self.__differs[handler.handled_type] = lambda a, b: handler.diff(
+            self, a, b)
+
 
 class Patcher(object):
     """Patch objects using nested diff."""
 
-    def __init__(self, patch_method=None):
-        """
-        Construct Patcher.
+    default_handler = _DEFAULT_HANDLER
 
-        Optional arguments:
-        `patch_method` method with such name, if patched object have one, will
-        be called with patch and current patcher as arguments. Patched object
-        expected for output.
-        Disabled (`None`) by default. This option is deprecated and will be
-        removed in the next release.
+    def __init__(self, handlers=()):
+        """
+        Initialize Patcher.
+
+        :param handlers: List of type handlers.
 
         """
-        self.__patch_method = patch_method
-
         self.__patchers = {
             dict: self.patch_dict,
             frozenset: self.patch_frozenset,
             list: self.patch_list,
             set: self.patch_set,
-            str: self.patch_multiline,
+            str: self.patch_text,
             tuple: self.patch_tuple,
         }
 
+        self.__handlers = {}
+
+        for handler in handlers:
+            self.set_handler(handler)
+
+        self.set_handler(nested_diff.handlers.TextHandler())
+
     def get_patcher(self, type_):
         """
         Return patch method for specified type.
@@ -449,6 +319,9 @@ class Patcher(object):
         :param type_: patched object type.
 
         """
+        warn('`get_patcher` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         try:
             return self.__patchers[type_]
         except KeyError:
@@ -465,24 +338,13 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        if self.__patch_method is not None:
-            warn('patch internal methods are deprecated and will be removed in'
-                 ' the next release', DeprecationWarning, stacklevel=2)
-            try:
-                return target.__getattribute__(self.__patch_method)(
-                    ndiff, self)
-            except AttributeError:
-                pass
-
+        # TODO: get rid of `get_patcher` and get funcs directly
         if 'D' in ndiff:
             return self.get_patcher(
                 ndiff['E' if 'E' in ndiff else 'D'].__class__,
             )(target, ndiff)
 
-        try:
-            return ndiff['N']
-        except KeyError:
-            return target
+        return self.default_handler.patch(self, target, ndiff)
 
     def patch_dict(self, target, ndiff):
         """
@@ -492,15 +354,10 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        for key, subdiff in ndiff['D'].items():
-            if 'D' in subdiff or 'N' in subdiff:
-                target[key] = self.patch(target[key], subdiff)
-            elif 'A' in subdiff:
-                target[key] = subdiff['A']
-            elif 'R' in subdiff:
-                del target[key]
+        warn('`patch_dict` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        return target
+        return _TYPE_HANDLERS[dict].patch(self, target, ndiff)
 
     def patch_list(self, target, ndiff):
         """
@@ -510,29 +367,14 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        i, j = 0, 0  # index, scatter
+        warn('`patch_list` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for subdiff in ndiff['D']:
-            if 'I' in subdiff:
-                i = subdiff['I'] + j
+        return _TYPE_HANDLERS[list].patch(self, target, ndiff)
 
-            if 'D' in subdiff or 'N' in subdiff:
-                target[i] = self.patch(target[i], subdiff)
-            elif 'A' in subdiff:
-                target.insert(i, subdiff['A'])
-                j += 1
-            elif 'R' in subdiff:
-                del target[i]
-                j -= 1
-                continue
-
-            i += 1
-
-        return target
-
-    def patch_multiline(self, target, ndiff):
+    def patch_text(self, target, ndiff):
         """
-        Return patched multiline string.
+        Return patched text (multiline string).
 
         Unlike GNU patch, this algorithm does not implement any heuristics and
         patch target in straightforward way: get position from hunk header and
@@ -542,31 +384,12 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        offset = 0
-        target = target.split('\n', -1)
+        warn('`patch_text` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for subdiff in ndiff['D']:
-            if 'I' in subdiff:  # hunk started
-                idx = subdiff['I'][0] + offset
-            elif 'A' in subdiff:
-                target.insert(idx, subdiff['A'])
-                offset += 1
-                idx += 1
-            elif 'R' in subdiff:
-                if target.pop(idx) != subdiff['R']:
-                    raise ValueError('Removing line does not match')
-                offset -= 1
-            elif 'U' in subdiff:
-                if target[idx] != subdiff['U']:
-                    raise ValueError('Unchanged line does not match')
-                idx += 1
-            else:
-                raise ValueError('Unsupported operation')
+        return self.__handlers[str].patch(self, target, ndiff)
 
-        return '\n'.join(target)
-
-    @staticmethod
-    def patch_set(target, ndiff):
+    def patch_set(self, target, ndiff):
         """
         Return patched set.
 
@@ -574,13 +397,10 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        for subdiff in ndiff['D']:
-            if 'A' in subdiff:
-                target.add(subdiff['A'])
-            elif 'R' in subdiff:
-                target.remove(subdiff['R'])
+        warn('`patch_set` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        return target
+        return _TYPE_HANDLERS[set].patch(self, target, ndiff)
 
     def patch_tuple(self, target, ndiff):
         """
@@ -590,7 +410,10 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        return tuple(self.patch_list(list(target), ndiff))
+        warn('`patch_tuple` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
+        return _TYPE_HANDLERS[tuple].patch(self, target, ndiff)
 
     def patch_frozenset(self, target, ndiff):
         """
@@ -600,7 +423,10 @@ class Patcher(object):
         :param ndiff: Nested diff.
 
         """
-        return frozenset(self.patch_set(set(target), ndiff))
+        warn('`patch_frozenset` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
+        return _TYPE_HANDLERS[frozenset].patch(self, target, ndiff)
 
     def set_patcher(self, type_, method):
         """
@@ -610,19 +436,36 @@ class Patcher(object):
         :param method: patch method.
 
         """
+        warn('`set_patcher` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         self.__patchers[type_] = method
 
+    def set_handler(self, handler):
+        """
+        Set handler.
+
+        :param handler: handlers.TypeHandler.
+
+        """
+        self.__patchers[handler.handled_type] = lambda a, b: handler.patch(
+            self, a, b)
+
 
 class Iterator(object):
     """Nested diff iterator."""
 
-    def __init__(self, sort_keys=False):
+    default_handler = _DEFAULT_HANDLER
+
+    def __init__(self, handlers=(), sort_keys=False):
         """
-        Construct iterator.
+        Initialize iterator.
 
         If `sort_keys` is `True`, then the output for mappings will be
         sorted by key. Disabled by default.
 
+        `handlers` is a list of type handlers.
+
         """
         self.sort_keys = sort_keys
 
@@ -632,15 +475,22 @@ class Iterator(object):
             tuple: self.iterate_sequence_diff,
         }
 
-    @staticmethod
-    def iterate__default(ndiff):
+        self.__handlers = {}
+
+        for handler in handlers:
+            self.set_handler(handler)
+
+    def iterate__default(self, ndiff):
         """
         Yield final diff (do not iterate deeper).
 
         :param ndiff: nested diff.
 
         """
-        yield ndiff, None, None
+        warn('`iterate__default` is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
+        yield from self.default_handler.iterate_diff(self, ndiff)
 
     def iterate_mapping_diff(self, ndiff):
         """
@@ -649,30 +499,22 @@ class Iterator(object):
         :param ndiff: nested diff.
 
         """
-        items = ndiff['D'].items()
+        warn('`iterate_mapping_diff` is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for key, subdiff in sorted(items) if self.sort_keys else items:
-            yield ndiff, key, subdiff
+        yield from _TYPE_HANDLERS[dict].iterate_diff(self, ndiff)
 
-    @staticmethod
-    def iterate_sequence_diff(ndiff):
+    def iterate_sequence_diff(self, ndiff):
         """
         Iterate over lists, tuples and alike nedsted diffs.
 
         :param ndiff: nested diff.
 
         """
-        idx = 0
+        warn('`iterate_sequence_diff` is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
 
-        for item in ndiff['D']:
-            try:
-                idx = item['I']
-            except KeyError:
-                pass
-
-            yield ndiff, idx, item
-
-            idx += 1
+        yield from _TYPE_HANDLERS[list].iterate_diff(self, ndiff)
 
     def get_iterator(self, ndiff):
         """
@@ -681,6 +523,9 @@ class Iterator(object):
         :param ndiff: nested diff.
 
         """
+        warn('`get_iterator` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         if 'E' in ndiff:
             return self.iterate__default(ndiff)
 
@@ -700,6 +545,9 @@ class Iterator(object):
         subdiff for this key.
 
         """
+        warn('`set_iterator` method is deprecated and will be removed in'
+             ' the next release', DeprecationWarning, stacklevel=2)
+
         self.__iterators[type_] = method
 
     def iterate(self, ndiff, depth=0):
@@ -710,6 +558,7 @@ class Iterator(object):
         :param depth: depth initial value (for use in subiterators).
 
         """
+        # TODO: get rid of `get_iterator` and get funcs directly
         stack = [self.get_iterator(ndiff)]
 
         while stack:
@@ -728,6 +577,16 @@ class Iterator(object):
             depth += 1
             stack.append(self.get_iterator(subdiff))
 
+    def set_handler(self, handler):
+        """
+        Set handler.
+
+        :param handler: handlers.TypeHandler.
+
+        """
+        self.__iterators[handler.handled_type] = \
+            lambda ndiff: handler.iterate_diff(self, ndiff)
+
 
 def diff(a, b, **kwargs):
     """
@@ -736,7 +595,7 @@ def diff(a, b, **kwargs):
     :param a: First object to diff.
     :param b: Second object to diff.
 
-    See `__init__` in Differ class for kwargs specification.
+    kwargs passed to Differ's constructor as is.
 
     """
     return Differ(**kwargs).diff(a, b)
@@ -749,7 +608,7 @@ def patch(target, ndiff, **kwargs):
     :param target: Object to patch.
     :param ndiff: Nested diff.
 
-    See `__init__` in Patcher class for kwargs specification.
+    kwargs passed to Patcher's constructor as is.
 
     """
     return Patcher(**kwargs).patch(target, ndiff)
