@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019-2021 Michael Samoglyadov
+# Copyright 2019-2022 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,6 +35,12 @@ class App(object):
     version = nested_diff.__version__
 
     def __init__(self, args=None):
+        """
+        Initialize app.
+
+        :param args: command line arguments; `sys.argv` used by default.
+
+        """
         self.override_excepthook()  # ASAP, but overridable by descendants
 
         self.argparser = self.get_argparser(description=self.__doc__)
@@ -49,12 +55,21 @@ class App(object):
         return json.loads(opts)
 
     def dump(self, file_, data, fmt):
+        """
+        Dump data using apropriate format.
+
+        :param file_: File object to dump.
+        :param data: Data to dump.
+        :param fmt: Format used for dump, one of `self.supported_ofmts`.
+
+        """
         self.get_dumper(
             fmt,
             **self._decode_fmt_opts(self.args.ofmt_opts)  # noqa C815
         ).dump(file_, data)
 
     def get_argparser(self, description=None):
+        """Return complete CLI argument parser."""
         return argparse.ArgumentParser(
             description=description,
             parents=(
@@ -64,6 +79,7 @@ class App(object):
         )
 
     def get_optional_args_parser(self):
+        """Return parser for optional part (dash prefixed) of CLI args."""
         parser = argparse.ArgumentParser(add_help=False)
         parser.add_argument(
             '--version',
@@ -106,10 +122,17 @@ class App(object):
 
     @staticmethod
     def get_positional_args_parser():
+        """Return parser for positional part (files etc) of CLI args."""
         return argparse.ArgumentParser(add_help=False)
 
     @staticmethod
     def get_dumper(fmt, **kwargs):
+        """
+        Return data dumper object based on desired format.
+
+        :param kwargs: passed to dumper's constructor as is.
+
+        """
         if fmt == 'json':
             return JsonDumper(**kwargs)
         elif fmt == 'yaml':
@@ -136,6 +159,12 @@ class App(object):
 
     @staticmethod
     def get_loader(fmt, **kwargs):
+        """
+        Return data loader object based on desired format.
+
+        :param kwargs: passed to loader's constructor as is.
+
+        """
         if fmt == 'json':
             return JsonLoader(**kwargs)
         elif fmt == 'yaml':
@@ -148,6 +177,12 @@ class App(object):
         raise RuntimeError('Unsupported input format: ' + fmt)
 
     def load(self, file_):
+        """
+        Load data from file using apropriate loader.
+
+        :param file_: File object to load from.
+
+        """
         if self.args.ifmt == 'auto':
             fmt = self.guess_fmt(file_, 'json')
         else:
@@ -172,6 +207,12 @@ class App(object):
         sys.excepthook = overrided
 
     def run(self):
+        """
+        App object entry point.
+
+        Must be implemented in derivatives.
+
+        """
         raise NotImplementedError
 
 
@@ -181,13 +222,34 @@ class Dumper(object):
     tty_final_new_line = False
 
     def encode(self, data):
+        """
+        Return encoded data.
+
+        :param data: data to encode.
+
+        Must be implemented in derivatives.
+
+        """
         raise NotImplementedError
 
     @staticmethod
     def get_opts(opts):
+        """
+        Return dumper options.
+
+        :param opts: initial options.
+
+        """
         return opts
 
     def dump(self, file_, data):
+        """
+        Encode and write data to file.
+
+        :param file_: File object.
+        :param data: data to write.
+
+        """
         file_.write(self.encode(data))
 
         if self.tty_final_new_line and file_.isatty():
@@ -200,70 +262,112 @@ class Loader(object):
     """Base class for data loaders."""
 
     def decode(self, data):
+        """
+        Return decoded data.
+
+        :param data: data to decode.
+
+        Must be implemented in derivatives.
+
+        """
         raise NotImplementedError
 
     @staticmethod
     def get_opts(opts):
+        """
+        Return loader options.
+
+        :param opts: initial options.
+
+        """
         return opts
 
     def load(self, file_):
+        """
+        Return decoded data loaded from file.
+
+        :param file_: File object.
+
+        """
         return self.decode(file_.read())
 
 
 class JsonDumper(Dumper):
-    """
-    JSON dumper.
-
-    All kwargs passed directly to `json.JSONEncoder`
-    `indent` is set to 3 and `sort_keys` to `True` if absent in kwargs
-
-    """
+    """JSON dumper."""
 
     tty_final_new_line = True
 
     def __init__(self, **kwargs):
+        """
+        Initialize dumper.
+
+        :param kwargs: options for `json.JSONEncoder`.
+
+        """
         super().__init__()
 
         import json
         self.encoder = json.JSONEncoder(**self.get_opts(kwargs))
 
     def encode(self, data):
+        """
+        Return JSON encoded data.
+
+        :param data: data to encode.
+
+        """
         return self.encoder.encode(data)
 
     @staticmethod
     def get_opts(opts):
+        """
+        Return dumper options.
+
+        :param opts: initial options.
+
+        `indent` is set to 3 and `sort_keys` to `True` if absent in opts.
+
+        """
         opts.setdefault('indent', 3)
         opts.setdefault('sort_keys', True)
         return opts
 
 
 class JsonLoader(Loader):
-    """
-    JSON loader.
-
-    All kwargs passed directly to `json.JSONDecoder`
-
-    """
+    """JSON loader."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize loader.
+
+        :param kwargs: options for `json.JSONDecoder`.
+
+        """
         super().__init__()
 
         import json
         self.decoder = json.JSONDecoder(**self.get_opts(kwargs))
 
     def decode(self, data):
+        """
+        Return data decoded from JSON.
+
+        :param data: data to decode.
+
+        """
         return self.decoder.decode(data)
 
 
 class IniDumper(Dumper):
-    """
-    INI dumper.
-
-    All kwargs passed directly to `configparser.ConfigParser`
-
-    """
+    """INI dumper."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize dumper.
+
+        :param kwargs: options for `configparser.ConfigParser`.
+
+        """
         super().__init__()
 
         import configparser
@@ -272,6 +376,12 @@ class IniDumper(Dumper):
         self.stringio = io.StringIO()
 
     def encode(self, data):
+        """
+        Return INI encoded data.
+
+        :param data: data to encode.
+
+        """
         self.encoder.read_dict(data)
         self.encoder.write(self.stringio)
 
@@ -279,20 +389,27 @@ class IniDumper(Dumper):
 
 
 class IniLoader(Loader):
-    """
-    INI loader.
-
-    All kwargs passed directly to `configparser.ConfigParser`
-
-    """
+    """INI loader."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize loader.
+
+        :param kwargs: options for `configparser.ConfigParser`.
+
+        """
         super().__init__()
 
         import configparser
         self.decoder = configparser.ConfigParser(**kwargs)
 
     def decode(self, data):
+        """
+        Return data decoded from INI.
+
+        :param data: data to decode.
+
+        """
         self.decoder.read_string(data)
 
         out = {}
@@ -311,12 +428,19 @@ class TomlDumper(Dumper):
     """TOML dumper."""
 
     def __init__(self, **kwargs):
+        """Initialize dumper."""
         super().__init__()
 
         import toml
         self.codec = toml
 
     def encode(self, data):
+        """
+        Return TOML encoded data.
+
+        :param data: data to encode.
+
+        """
         return self.codec.dumps(data)
 
 
@@ -324,54 +448,94 @@ class TomlLoader(Loader):
     """TOML loader."""
 
     def __init__(self, **kwargs):
+        """Initialize loader."""
         super().__init__()
 
         import toml
         self.codec = toml
 
     def decode(self, data):
+        """
+        Return data decoded from TOML.
+
+        :param data: data to decode.
+
+        """
         return self.codec.loads(data)
 
 
 class YamlDumper(Dumper):
-    """
-    YAML dumper.
-
-    All kwargs passed directly to `yaml.safe_dump()`
-    `default_flow_style` is set to `False` if absent in kwargs
-
-    """
+    """YAML dumper."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize dumper.
+
+        :param kwargs: options for `yaml.dump`.
+
+        """
         super().__init__()
 
         import yaml
-        self.codec = yaml
-        self.codec_opts = self.get_opts(kwargs)
+        try:
+            from yaml import CSafeDumper as YamlDumper
+        except ImportError:
+            from yaml import SafeDumper as YamlDumper
+
+        self.yaml = yaml
+        self.yaml_dumper = YamlDumper
+        self.opts = self.get_opts(kwargs)
 
     def encode(self, data):
-        return self.codec.safe_dump(data, **self.codec_opts)
+        """
+        Return YAML encoded data.
+
+        :param data: data to encode.
+
+        """
+        return self.yaml.dump(data, Dumper=self.yaml_dumper, **self.opts)
 
     @staticmethod
     def get_opts(opts):
+        """
+        Return dumper options.
+
+        :param opts: initial options.
+
+        `default_flow_style` is set to `False` if absent in opts.
+
+        """
         opts.setdefault('default_flow_style', False)
         return opts
 
 
 class YamlLoader(Loader):
-    """
-    YAML loader.
-
-    All kwargs passed directly to `yaml.safe_load()`
-
-    """
+    """YAML loader."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize loader.
+
+        :param kwargs: options for `yaml.safe_load`.
+
+        """
         super().__init__()
 
         import yaml
-        self.codec = yaml
-        self.codec_opts = self.get_opts(kwargs)
+        try:
+            from yaml import CSafeLoader as YamlLoader
+        except ImportError:
+            from yaml import SafeLoader as YamlLoader
+
+        self.yaml = yaml
+        self.yaml_loader = YamlLoader
+        self.opts = self.get_opts(kwargs)
 
     def decode(self, data):
-        return self.codec.safe_load(data, **self.codec_opts)
+        """
+        Return data decoded from YAML.
+
+        :param data: data to decode.
+
+        """
+        return self.yaml.load(data, Loader=self.yaml_loader, **self.opts)
