@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019-2021 Michael Samoglyadov
+# Copyright 2019-2022 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import sys
 
 import nested_diff
 import nested_diff.cli
+import nested_diff.handlers
 
 
 class App(nested_diff.cli.App):
@@ -40,7 +41,6 @@ class App(nested_diff.cli.App):
 
         """
         diff_opts = {
-            'text_diff_ctx': self.args.text_ctx,
             'A': self.args.A,
             'N': self.args.N,
             'O': self.args.O,  # noqa: E741
@@ -49,9 +49,22 @@ class App(nested_diff.cli.App):
         }
         diff_opts.update(kwargs)
 
-        return nested_diff.diff(a, b, **diff_opts)
+        if diff_opts['R'] == 'trim':
+            diff_opts['R'] = True
+            diff_opts['trimR'] = True
+        else:
+            diff_opts['R'] = int(diff_opts['R'])
+
+        differ = nested_diff.Differ(**diff_opts)
+
+        if self.args.text_ctx >= 0:
+            differ.set_handler(nested_diff.handlers.TextHandler(
+                context=self.args.text_ctx))
+
+        return differ.diff(a, b)
 
     def get_optional_args_parser(self):
+        """Return parser for optional part (dash prefixed) of CLI args."""
         parser = super().get_optional_args_parser()
 
         parser.add_argument(
@@ -78,14 +91,16 @@ class App(nested_diff.cli.App):
                             help="show item's new values; enabled by default")
         parser.add_argument('-O', type=int, choices=(0, 1), default=1,
                             help="show item's old values; enabled by default")
-        parser.add_argument('-R', type=int, choices=(0, 1), default=1,
-                            help='Show removed items; enabled by default')
+        parser.add_argument('-R', choices=('0', '1', 'trim'), default=1,
+                            help='Show removed items; enabled (1) by default. '
+                            'Value will be replaced by null when "trim" used')
         parser.add_argument('-U', type=int, choices=(0, 1), default=0,
                             help='show unchanged items; disabled by default')
 
         return parser
 
     def get_positional_args_parser(self):
+        """Return parser for positional part (files etc) of CLI args."""
         parser = super().get_positional_args_parser()
 
         parser.add_argument('file1', type=argparse.FileType())
@@ -94,6 +109,12 @@ class App(nested_diff.cli.App):
         return parser
 
     def get_dumper(self, fmt, **kwargs):
+        """
+        Return data dumper object based on desired format.
+
+        :param kwargs: passed to dumper's constructor as is.
+
+        """
         if fmt == 'auto':
             if self.args.out.isatty():
                 fmt = 'term'
@@ -110,6 +131,7 @@ class App(nested_diff.cli.App):
         return super().get_dumper(fmt, **kwargs)
 
     def run(self):
+        """Diff app object entry point."""
         diff = self.diff(
             self.load(self.args.file1),
             self.load(self.args.file2),
@@ -122,13 +144,27 @@ class App(nested_diff.cli.App):
 
 
 class AbstractFmtDumper(nested_diff.cli.Dumper):
-    """Base class for diff dumpers."""
+    """Base class for diff formatters dumpers."""
 
     def encode(self, data):
+        """
+        Return encoded (formatted) diff.
+
+        :param data: diff data to format.
+
+        """
         return self.encoder.format(data)
 
     @staticmethod
     def get_opts(opts):
+        """
+        Return dumper options.
+
+        :param opts: initial options.
+
+        `sort_keys` is set to `True` if absent in opts.
+
+        """
         opts.setdefault('sort_keys', True)
         return opts
 
@@ -136,16 +172,23 @@ class AbstractFmtDumper(nested_diff.cli.Dumper):
 class HtmlDumper(AbstractFmtDumper):
     """Human friendly HTML dumper for nested diff."""
 
-    def __init__(self, **kwargs):
+    def __init__(self, html_opts=(('lang', 'en'), ('title', 'Nested diff')),
+                 **kwargs):
+        """
+        Initialize dumper.
+
+        :param html_opts: may contain `header` (default is brief HTML5
+        boilerplate) and `footer` (page closing tags). Also `lang` and `title`
+        supported which define according values for default `header`.
+
+        Rest kwargs passed to `fmt.HtmlFormatter`.
+
+        """
         super().__init__()
         from html import escape
         from nested_diff import fmt
-        self.html_opts = {
-            'lang': 'en',
-            'title': 'Nested diff',
-        }
-        self.html_opts.update(kwargs.pop('html_opts', {}))
 
+        self.html_opts = dict(html_opts)
         self.formatter = fmt.HtmlFormatter(**self.get_opts(kwargs))
 
         if 'header' not in self.html_opts:
@@ -159,6 +202,12 @@ class HtmlDumper(AbstractFmtDumper):
             self.html_opts['footer'] = '</body></html>'
 
     def encode(self, data):
+        """
+        Return diff formatted as HTML.
+
+        :param data: diff to format.
+
+        """
         return self.formatter.format(
             data,
             header=self.html_opts['header'],
@@ -170,6 +219,12 @@ class TermDumper(AbstractFmtDumper):
     """Same as TextDumper but with ANSI term colors."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize dumper.
+
+        :param kwargs: passed to `fmt.TermFormatter`.
+
+        """
         super().__init__()
         from nested_diff import fmt
         self.encoder = fmt.TermFormatter(**self.get_opts(kwargs))
@@ -179,6 +234,12 @@ class TextDumper(AbstractFmtDumper):
     """Human friendly text dumper for nested diff."""
 
     def __init__(self, **kwargs):
+        """
+        Initialize dumper.
+
+        :param kwargs: passed to `fmt.TextFormatter` as is.
+
+        """
         super().__init__()
         from nested_diff import fmt
         self.encoder = fmt.TextFormatter(**self.get_opts(kwargs))
